'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.withApiRedux = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactRedux = require('react-redux');

var _actions = require('../actions');

var _constants = require('../constants');

var _selectors = require('../selectors');

var _utils = require('./utils');

var _init = require('../init');

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var withApiRedux = exports.withApiRedux = function withApiRedux(WrappedComponent) {
  var _defaultProps;

  var propTypes = {
    onFetchingChange: _propTypes2.default.func
  };

  var defaultProps = (_defaultProps = {}, _defineProperty(_defaultProps, _constants.BACKEND_ACTIONS.CREATE, _init.initCreate), _defineProperty(_defaultProps, _constants.BACKEND_ACTIONS.NEW, _init.initNew), _defineProperty(_defaultProps, _constants.BACKEND_ACTIONS.UPDATE, _init.initUpdate), _defineProperty(_defaultProps, _constants.BACKEND_ACTIONS.EDIT, _init.initEdit), _defineProperty(_defaultProps, _constants.BACKEND_ACTIONS.SHOW, _init.initShow), _defineProperty(_defaultProps, _constants.BACKEND_ACTIONS.DESTROY, _init.initDestroy), _defineProperty(_defaultProps, _constants.BACKEND_ACTIONS.INDEX, _init.initIndex), _defaultProps);

  /**
   * HOC Component that wrapps api actions and state for any component.
   *
   * @author [César Parra](https://github.com/cparram)
   */

  var WithApiRedux = function (_React$Component) {
    _inherits(WithApiRedux, _React$Component);

    function WithApiRedux() {
      _classCallCheck(this, WithApiRedux);

      return _possibleConstructorReturn(this, (WithApiRedux.__proto__ || Object.getPrototypeOf(WithApiRedux)).apply(this, arguments));
    }

    _createClass(WithApiRedux, [{
      key: 'componentDidMount',
      value: function componentDidMount() {
        var _wrapped = this.wrapped,
            apiReduxNew = _wrapped.apiReduxNew,
            apiReduxIndex = _wrapped.apiReduxIndex,
            apiReduxShow = _wrapped.apiReduxShow,
            apiReduxEdit = _wrapped.apiReduxEdit;


        apiReduxNew && apiReduxNew();
        apiReduxIndex && apiReduxIndex();
        apiReduxShow && apiReduxShow();
        apiReduxEdit && apiReduxEdit();
      }
    }, {
      key: 'componentDidUpdate',
      value: function componentDidUpdate(prevProps, _prevState) {
        var _this2 = this;

        var onFetchingChange = this.props.onFetchingChange;


        var actions = [{ action: 'New' }, { action: 'Create' }, { action: 'Update' }, { action: 'Show' }, { action: 'Edit' }, { action: 'Destroy' }, { action: 'Index', scope: 'Records' }];

        var fetching = this.isFetching(actions, this.props);
        var prevFetching = this.isFetching(actions, prevProps);

        if (fetching !== prevFetching) onFetchingChange && onFetchingChange(fetching);

        actions.forEach(function (_ref) {
          var action = _ref.action,
              _ref$scope = _ref.scope,
              scope = _ref$scope === undefined ? 'Record' : _ref$scope;

          var apiAction = _this2.props[_constants.BACKEND_ACTIONS[action.toUpperCase()]];
          var prevAction = prevProps[_constants.BACKEND_ACTIONS[action.toUpperCase()]];

          if (apiAction.fetching === prevAction.fetching || apiAction.fetching) return;

          var method = 'onApiRedux' + action;
          _this2.wrapped[method] && _this2.wrapped[method](apiAction.error || apiAction[scope.toLowerCase()]);
        });
      }
    }, {
      key: 'isFetching',
      value: function isFetching(actions, props) {
        return actions.reduce(function (acc, _ref2) {
          var action = _ref2.action;

          var apiAction = props[_constants.BACKEND_ACTIONS[action.toUpperCase()]];
          return acc || apiAction.fetching;
        }, false);
      }
    }, {
      key: 'render',
      value: function render() {
        var _this3 = this;

        return _react2.default.createElement(WrappedComponent, _extends({
          ref: function ref(_ref3) {
            return _this3.wrapped = _ref3;
          },
          apiReduxRef: function apiReduxRef(ref) {
            return _this3.wrapped = ref;
          }
        }, this.props));
      }
    }]);

    return WithApiRedux;
  }(_react2.default.Component);

  WithApiRedux.displayName = 'WithApiRedux(' + (0, _utils.getDisplayName)(WrappedComponent) + ')';
  WithApiRedux.propTypes = propTypes;
  WithApiRedux.defaultProps = defaultProps;

  return WithApiRedux;
};

exports.default = function (WrappedComponent) {

  var mapStateToProps = _selectors.getApiReduxByResource;

  var mapDispatchToProps = function mapDispatchToProps(dispatch, _ref4) {
    var apiResource = _ref4.apiResource;
    return {
      apiNewRequest: function apiNewRequest(opts) {
        return dispatch((0, _actions.apiNew)(_extends({ apiResource: apiResource }, opts)));
      },
      apiCreateRequest: function apiCreateRequest(opts) {
        dispatch((0, _actions.apiCreate)(_extends({ apiResource: apiResource }, opts)));
      },
      apiUpdateRequest: function apiUpdateRequest(opts) {
        dispatch((0, _actions.apiUpdate)(_extends({ apiResource: apiResource }, opts)));
      },
      apiIndexRequest: function apiIndexRequest(opts) {
        dispatch((0, _actions.apiIndex)(_extends({ apiResource: apiResource }, opts)));
      },
      apiShowRequest: function apiShowRequest(opts) {
        dispatch((0, _actions.apiShow)(_extends({ apiResource: apiResource }, opts)));
      },
      apiEditRequest: function apiEditRequest(opts) {
        dispatch((0, _actions.apiEdit)(_extends({ apiResource: apiResource }, opts)));
      },
      apiDestroyRequest: function apiDestroyRequest(opts) {
        dispatch((0, _actions.apiDestroy)(_extends({ apiResource: apiResource }, opts)));
      },
      clearAction: function clearAction(apiAction, apiResource) {
        dispatch((0, _actions.clear)(apiAction, apiResource));
      }
    };
  };

  var component = withApiRedux(WrappedComponent);

  return (0, _reactRedux.connect)(mapStateToProps, mapDispatchToProps, null, { withRef: true })(component);
};